/*
---------------------------------------------------------------------------
 Algo10-2.c   C program for implementing Algorithm 10.2
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 10.2 (Forward-Difference Method for the Heat Equation).
 Section	10.2, Parabolic Equations, Page 516
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 10.2 (Forward-Difference Method for the Heat Equation)

                                                2
 To approximate the solution of u_(t)(x,t)  =  c u_(x)(x,t)

 over R = {(x,t): 0 <= x <= a, 0 <= t <= b} with 

 u(x,0) = f(x)  for  0 <= x <= a and 

 u(0,t) = c_1,  u(a,t) = c_2  for  0 <= t <= b.

 User has to supply function Fi(gridpoint). 
 An example is implemented in this program.

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define Max 50

/* Global Variables */

double H;                /* Step size  */

/* Prototypes */

double Fi(int i);

/* Grid function for amplitude */

double Fi(int i)
{
    extern double H;
    double arg;

    arg = H * (i - 1);

    if( (arg >= 0) && (arg <= 1.0) ) return (4.0 * arg - 4.0 * arg * arg);
    else
    {
        printf(" Fi() :Argument not in range ! Exiting !\n");
        printf(" arg  : %lf\n", arg);
        exit(0);
    }
}

/* -------------------------------------------------------- */

/*  Main program for algorithm 10-2  */

/*  Remember : in C the fields start with index 0.
    In this program this field is ignored and when you
    change the program be aware of the index 0 in the U[Max][Max]
    array ! Here - as said before - it is ignored, so don't
    access it if you do not initialize it. */

void main(void)

{
    int i, j;                /* Loop Counters                         */
    double A, B;             /* INPUT :Width and height of Rectangle  */
    double C;                /* INPUT : Wave equation const.          */
    int N, M;                /* Dimensions of the grid                */
    double K, S, C1, C2, R;
    double U[Max][Max];      /* Grid-Amplitudes                       */

    printf("-- Forward-Difference Method for the Heat Equation ----\n");
    printf("--------- Example 10.3 on page 511 --------------------\n");
    printf("-------------------------------------------------------\n");

    printf("Please enter A (interval boundary of x)\n");
    printf("For present example type : 1.0\n");
    scanf("%lf",&A);
    printf("Please enter B (interval boundary of t)\n");
    printf("For present example type : 0.2\n");
    scanf("%lf",&B);
    printf("You entered A = %lf and B = %lf \n", A, B);
    printf("-------------------------------------------------------\n");
    printf("\n");
    printf("Please enter dimension of grid in x-direction ( < 50 ):\n");
    printf("For present example type : 6\n");
    scanf("%d",&N);
    if ( N >= 50 ) 
    {
       printf("MUST BE SMALLER THAN 50. ... terminating.\n");
       exit(0);
    }
    printf("Please enter dimension of grid in y-direction ( < 50 ):\n");
    printf("For present example type : 11\n");
    scanf("%d",&M);
    if ( M >= 50 ) 
    {
       printf("MUST BE SMALLER THAN 50. ... terminating.\n");
       exit(0);
    }
    printf("You entered N = %d and M = %d\n", N, M);
    printf("-------------------------------------------------------\n");
    printf("\n");
    printf("Please enter the heat-equation constant C :\n");
    printf("For present example type : 1\n");
    scanf("%lf",&C);
    printf("You entered C = %lf\n", C);
    printf("------------------------------------------------\n");
    printf("\n");
    printf("Please enter constant C1 = u(0,t) :\n");
    printf("For present example type : 0\n");
    scanf("%lf",&C1);
    printf("You entered C1 = %lf\n", C1);
    printf("------------------------------------------------\n");
    printf("\n");
    printf("Please enter constant C2 = u(A,t) :\n");
    printf("For present example type : 0\n");
    scanf("%lf",&C2);
    printf("You entered C2 = %lf\n", C2);
    printf("------------------------------------------------\n");
    printf("\n");


    /* Compute step sizes */

    H   = A / ( N - 1 );
    K   = B / ( M - 1 );
    R   = C * C * K / ( H * H );
    S   = 1.0 - 2.0 * R;

    /* Boundary conditions */

    for ( j = 1; j <= M; j++ )
    {
        U[1][j] = C1;
        U[N][j] = C2;
    }

    /* First row */

    for ( i = 2; i <= N - 1 ; i++ ) U[i][1] = Fi(i);

    /* Generate new waves */

    for ( j = 2; j <= M; j++ )
    {
        for ( i = 2; i <= N - 1; i++ )
        {
            U[i][j] = S * U[i][j-1] + R * ( U[i-1][j-1] + U[i+1][j-1] );
        }
    }
     
    /* Output the solution */

    printf("The grid amplitudes look like this :\n");
    printf("\n");

    for ( j = 1; j <= M; j++ )
    {
        printf("%8.6lf ", K * (j - 1));

        for ( i = 1; i <= N; i++ ) printf(" %8.6lf ", U[i][j]);

        printf("\n");
    }

}   /* End of main program */

