/*
---------------------------------------------------------------------------
 Algo11-3.c   C program for implementing Algorithm 11.3
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 11.3 (Jacobi Iteration for Eigenvalues and Eigenvectors).
 Section   11.3, Jacobi's Method, Page 571
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 11.3 (Jacobi Iteration for eigenvalues and Eigenvectors)

                                                      n
 To compute the full set of eigenpairs {Lambda_j, V_j}    
                                                      j=1

 of the n x n real symmetric matrix A. Jacobi iteration is
 used to find all eigenpairs.

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>


#define MaxO 20

/* Prototypes */

void INITIALIZE(double V[MaxO][MaxO], int N);
double RMS(double A[MaxO][MaxO], int N);
void ZEROOUT(double A[MaxO][MaxO], double V[MaxO][MaxO], int N, int P, int Q);

/* Global Variables */

double V[MaxO][MaxO];
double A[MaxO][MaxO];
int P, Q;


/* -------------------------------------------------------- */

/*  Main program for algorithm 11.3  */

void main(void)

{
    extern void INITIALIZE();
    extern double RMS();
    extern void ZEROOUT();

    double Epsilon = 1E-07;        /* Off-diagonal element Tolerance */
    double MaxSweep = 50;          /* Maximum number of sweeps       */
    int CountS, State;
    double T;
    int       i, j;                /* Loop Counters                  */
    int N;                         /* Order of Matrix                */


    printf("-- Jacobi Iteration for Eigenvalues and Eigenvectors --\n");
    printf("--------- Try Example 11.7 on page 569 ----------------\n");
    printf("-------------------------------------------------------\n");

    printf("Please enter order of Matrix  A ( < %d !)\n", MaxO + 1);
    scanf("%d",&N);
    if( N > MaxO )
    {
       printf(" Order of matrix must be less than %d\n",MaxO+1);
       printf(" Terminating. Sorry\n");
       exit(0);
    }
    printf("Please enter elements of matrix  A  row by row:\n");

    for ( i = 0; i < N; i++)
    {
        for ( j = 0; j < N; j++)
        {
            printf(" Enter Element No. %d of row %d\n", j+1, i+1);
            scanf("%lf", &A[i][j]);
            printf("You entered A[%d][%d]= %lf\n", i+1, j+1, A[i][j] );
            printf("------------------------------------------------\n");
        }
    }

    printf("\n");

    CountS = 0;
    
    INITIALIZE(V, N);  

    do
    {
      T = RMS(A, N);

      CountS++;
      State = 1;
      
      for( P = 0; P < N - 1; P++ )
      {
         for( Q = P + 1; Q < N; Q++ ) 
         {
            if ( (fabs(A[P][Q]) / T) > Epsilon )
            {
               ZEROOUT(A, V, N, P, Q);
               State = 0;
            }
         }
      }            

      } while ( (State != 1) || (MaxSweep > CountS) );

/* Output */

    for ( i = 0; i < N; i++)
    {
        printf(" Eigen-Vector No %d :\n", i+1);
        printf("------------------------------------------------\n");
        for ( j = 0; j < N; j++) printf(" %lf ",  V[j][i] );
        printf("\n================================================\n");
    }

    printf("\n");
    printf(" Eigen-Values are :\n");
    for ( i = 0; i < N; i++) printf(" Lambda_%d = %lf\n", i+1, A[i][i]);

}  /* End of main */


/* Subroutine INITIALIZE */

void INITIALIZE(double V[MaxO][MaxO], int N)
{
   int J, K;   /* Loop Counters */

   /* Initialize the matrix V as the identity matrix */

   for ( J = 0; J < N ; J++)
   {
      for ( K = 0; K < N; K++)
      {  
         if ( J != K ) V[J][K] = 0.0;
         else          V[J][K] = 1.0;
      }
   }
   return ;

}    /* End of subroutine INITIALIZE */


/*  Subroutine RMS   */

double RMS(double A[MaxO][MaxO], int N)
{
   double Sum = 0;
   int J;
   
   for ( J = 0; J < N; J++) Sum += A[J][J] * A[J][J];
   return ( sqrt(Sum/ (double)(N)) );
}   
   
void ZEROOUT(double A[MaxO][MaxO], double V[MaxO][MaxO], int N, int P, int Q)
{
   double Theta;
   double T, C, S;
   int J;
   double XP[MaxO], YP[MaxO];
   double XQ[MaxO], YQ[MaxO];

   /* determine the angle of rotation */
   
   Theta = ( A[Q][Q] - A[P][P] ) / ( 2.0 * A[P][Q] );
   T     = ( fabs(Theta) + sqrt(Theta * Theta + 1.0) );
   if ( Theta >= 0 ) T =  1.0 / T;
   else              T = -1.0 / T;

   /* compute C and S from angle of rotation */

   C = 1.0 / sqrt(T * T + 1.0);
   S = C * T;

   /* Store column P and Q of matrix A */

   for (J = 0; J < N; J++)
   {
     XP[J] = A[J][P];
     XQ[J] = A[J][Q];
   }


   /*                     T           */
   /* construct  D_  =  R_  A  R      */
   /*              1      1     1     */

   A[P][Q] = 0;
   A[Q][P] = 0;
   A[P][P] = C * C * XP[P] + S * S * XQ[Q] - 2.0 * C * S * XQ[P];
   A[Q][Q] = S * S * XP[P] + C * C * XQ[Q] + 2.0 * C * S * XQ[P];

   for ( J = 0; J < N; J++ )
   {
      if ( (J != P)  && (J != Q) )
      {
         A[J][P] = C * XP[J] - S * XQ[J];
         A[P][J] = A[J][P];
         A[J][Q] = C * XQ[J] + S * XP[J];
         A[Q][J] = A[J][Q];
      }
   }  
   
   for ( J = 0; J < N; J++ )
   {
       YP[J] = V[J][P];
       YQ[J] = V[J][Q];
   }

   /* update the matrix of Eigen-vectors */

   for ( J = 0; J < N; J++ )
   {
      V[J][P] = C * YP[J] - S * YQ[J];
      V[J][Q] = S * YP[J] + C * YQ[J];
   }

   return ;

}   /* End of subroutine ZEROOUT */ 

