/*
---------------------------------------------------------------------------
 Algo11-4.c   C program for implementing Algorithm 11.4
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 11.4 (Reduction to Tridiagonal Form).
 Section   11.4, Eigenvalues of Symmetric Matrices, Page 581
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 11.4 (Reduction to Tridiagonal Form)

 To reduce the  n x n symmetric matrix A to tridiagonal
 form by using  n - 2 Householder transformation. 

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define MaxOrder 50

/* -------------------------------------------------------- */

/*  Main program for algorithm 11.4  */

void main(void)

{
    int K, I, J;               /* Loop counter                   */
    int N;                     /* Order of Matrix                */
    double W[MaxOrder], V[MaxOrder], Q[MaxOrder];
    double A[MaxOrder][MaxOrder]; /* Matrix                      */
    double Sum, S, R, C;


    printf("-------Reduction to Tridiagonal Form ------------\n");
    printf("--------- Example 11.8 on page 580 --------------\n");
    printf("-------------------------------------------------\n");

    printf("Please enter order of Matrix  A ( < %d !)\n", MaxOrder);
    scanf("%d",&N);
    if( N > MaxOrder )
    {
    printf(" Order of Matrix must be less than %d\n",MaxOrder);
    printf(" Terminating. Sorry\n");
    exit(0);
    }
    printf("-----------------------------------------------------------\n");
    printf("Enter upper and diagonal elements of matrix  A  row by row:\n");
    printf("Matrix is assumed to be symmetric !\n");


    /* In C arrays start with 0 . Here we keep the A[0][0] element
       as a dummy doing nothing. For security we initialize it though. */

    A[0][0] =  1.0;  /* dummy field : is not used by algorithm here */

    for ( I = 1; I <= N; I++)
    {
        for ( J = I; J <= N; J++)
        {
            printf(" Enter Element No. %d of row %d\n", J, I);
            scanf("%lf", &A[I][J]);
            /* mirror the input to the lower triangle of the matrix */
            if( I != J ) A[J][I] = A[I][J];
            printf("You entered A[%d][%d]= %lf\n", I, J, A[I][J] );
            printf("------------------------------------------------\n");
        }
    }

    printf("\n");

        printf("------- Matrix  A on INPUT --------\n");

    for ( J = 1; J<= N; J++)
    {
        for ( I = 1; I <= N; I++)
        {
            printf("%12.6lf", A[I][J]);
        }
     printf("\n");
    }

    printf("---------------------------------------\n");

    for ( K = 1; K <= N - 2; K++ )
    {

    /*-----------------------------------------------*/
    /* Do what Procedure FormW in the book does      */
    /*-----------------------------------------------*/

    /* Construct the vector W */
    Sum = 0;
    for (J = K+1; J <= N; J++) Sum += A[J][K] * A[J][K];
    S   = sqrt(Sum);

    if ( A[K+1][K] < 0 ) S = -S;
    R = sqrt( 2.0 * A[K+1][K] * S + 2.0 * S * S );

    /* First k elements are zero */

    for (J = 1; J <= K; J++) W[J] = 0.0;

    W[K+1] = ( A[K+1][K] + S ) / R;

    for (J = K + 2; J <= N; J++) W[J] = A[J][K] / R;

    /*-----------------------------------------------*/
    /* Do what Procedure FormV in the book does      */
    /*-----------------------------------------------*/
    
    /* Construct the vector V */
    
    /* First k elements are zero */

    for (I = 1; I <= K; I++) V[I] = 0.0;

    for (I = K+1; I <= N; I++) 
    {
        Sum = 0.0;
        for (J = K+1; J <= N; J++) Sum += A[I][J] * W[J];

        V[I] = Sum;
    }            

    /*-----------------------------------------------*/
    /* Do what Procedure FormQ in the book does      */
    /*-----------------------------------------------*/

    /* Construct the vector Q */

    C = 0;
    for (J = K+1; J <= N; J++) C += W[J] * V[J];

    /* First k elements are zero */

    for (J = 1; J <= K; J++)   Q[J] = 0.0;
    for (J = K+1; J <= N; J++) Q[J] = V[J] - C * W[J];

    /*-----------------------------------------------*/
    /* Do what the Procedure FormA in the book does  */
    /*-----------------------------------------------*/

    for (J = K + 2; J <= N; J++) A[J][K] = A[K][J] = 0.0;

    A[K+1][K] = A[K][K+1] = -S;

    for (J = K; J <= N; J++) A[J][J] -= 4.0 * Q[J] * W[J];

    for (I = K+1; I <= N; I++) 
    {
        for (J = I+1; J <= N; J++) 
        {
            A[I][J] = A[I][J] - 2.0 * W[I] * Q[J] - 2.0 * Q[I] * W[J];
            A[J][I] = A[I][J];
        }
    }

    } /* End of the exterior main FOR-Loop */

    /* Output the matrix A */

    printf("--- Output : tridiagonal matrix  A ---------\n");

    for ( J = 1; J<= N; J++)
    {
        for ( I = 1; I <= N; I++)
        {
            printf("%12.6lf", A[I][J]);
        }
     printf("\n");
    }

    printf("---------------------------------------\n");

}   /* End of main program */

