/*
---------------------------------------------------------------------------
 Algo2-6.c   C programs for implementing Algorithm 2.6
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 2.6 (Secant Method).
 Section   2.4, Newton-Raphson and Secant Methods, Page 85
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------

 Algorithm 2.6 (Secant Method). To find a root of  f(x) = 0
 given two initial approximation  p_0  and  p_1  using the iteration

                           f(p_k) [p_k - p_(k-1)]
      p_(k+1)  =  p_(k) -  ----------------------    for k = 1, 2, ...
                           f(p_k) - f(p_(k-1))

---------------------------------------------------------------------------
*/

/* User has to supply a function named :  ffunction

   An example is included in this program     */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>


/*  define prototype for USER-SUPPLIED function f(x)  */

    double ffunction(double x);


/*  EXAMPLE for "ffunction"   */

    double ffunction(double x)

    {
        return ( pow(x,3) - 3 * x + 2 );
    }


/* -------------------------------------------------------- */

/*  Main program for algorithm 2.6  */

    void main()

{
    double Delta   = 1E-6;       /* Tolerance  */
    double Epsilon = 1E-6;       /* Tolerance  */
    double Small   = 1E-6;       /* Tolerance  */

    int Max = 149;  /* Maximum number of iterations  */
    int Cond = 0;   /* Condition fo loop termination */
    int K;          /* Counter for loop              */

    double P0, P1;    /* INPUT : Must be close to the root */
    double P2;        /* New iterate    */
    double Y0;        /* Function value */
    double Y1;        /* Function value */
    double Y2;        /* Function value */
    double Df;        /* Slope          */
    double Dp;
    double RelErr;


    printf("----------------------------------------------\n");
    printf("Please enter initial values of root !\n");
    printf("EXAMPLE :  -2.6 -2.4\n");
    scanf("%lf %lf",&P0, &P1);
    printf("----------------------------------------------\n");
    printf("The initial values are: %lf %lf\n",P0, P1);

    Y0 = ffunction(P0);
    Y1 = ffunction(P1);

    for ( K = 1; K <= Max ; K++) {

        if(Cond) break;

        Df = (Y1 - Y0) / (P1 - P0);      /* Compute the slope      */

         if( Df == 0) {                  /* Check division by zero */
             Cond = 1;
             Dp   = 0;
         }

         else Dp = Y1/Df;

        P2 = P1 - Dp;         /* New iterate        */
        Y2 = ffunction(P2);   /* New function value */

        RelErr = 2 * fabs(Dp) / ( fabs(P2) + Small );  /* Relative error */

        if( (RelErr < Delta)  && (fabs(Y2) < Epsilon) ) { /* Check for   */

            if( Cond != 1) Cond = 2;                      /* convergence */

        }

        P0 = P1;
        P1 = P2;
        Y0 = Y1;
        Y1 = Y2;
    }

    printf("----------------------------------------------\n");
    printf("The current %d -th iterate is %lf\n",K, P2);
    printf("Consecutive iterates differ by %lf\n",Dp);
    printf("The value of f(x) is %lf\n",Y2);
    printf("----------------------------------------------\n");

if(Cond == 0) printf("The maximum number of iterations was exceeded !\n");

if(Cond == 1) printf("Division by zero was encountered !\n");

if(Cond == 2) printf("The root was found with the desired tolerance !\n");

    printf("----------------------------------------------\n");

}   /* End of main program */

