/*
---------------------------------------------------------------------------
 Algo3-2.c   C program for implementing Algorithm 3.2
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 3.2 (Upper-Triangularization Followed by Back Substitution).
 Section   3.4, Gaussian Elimination and Pivoting, Page 156
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------

 Algorithm 3.2 ( Upper Triangularization Followed by Back Substitution).

 To construct the solution to  AX = B, by first reducing the
 augmented matrix  [A,B]  to upper-triangular form and the
 performing back substitution.

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

/* -------------------------------------------------------- */

/*  Main program for algorithm 3.2  */

/*  remember : in C the fields begin with element 0  */

#define Limit  20

void main(void)
{
    int K, P, C, J;                  /* Loop counters               */
    double A[Limit][Limit+1];        /* A in  AX = B ,  INPUT       */
    int N;                           /* Number of equations         */   
    int    Row[Limit];               /* Field with row-number       */
    double X[Limit];                 /* Solution-vector             */
    double SUM, M;                                                       
    int  T;

    printf("Try the examples on page 157.. of the book !\n");
    printf("------------------------------------------\n");
    do  /* Force proper INPUT */
    {
      printf("Please enter number of equations [Not more than %d]\n",Limit);
      scanf("%d", &N);
    } while( N > Limit);


    printf("You say there are %d equations.\n", N);

    printf("-----------------------------------------------------\n");
    printf("From  AX = B enter elements of [A,B] row by row:\n");
    printf("[A,B] is the augmented matrix:\n");
    printf("-----------------------------------------------------\n");

    for (K = 1; K <= N; K++)
    {
        for (J = 1; J <= N+1; J++)
        {
             printf(" For row %d enter element %d please :\n", K, J);
             scanf("%lf", &A[K-1][J-1]); 
             printf(" You entered : %lf \n", A[K-1][J-1] );
             printf("------------------------------------------\n");
        }
    }


    /* Initialize the pointer vector */

    for (J = 1; J<= N; J++) Row[J-1] = J - 1;

    /* Start upper-triangularization */

    for (P = 1; P <= N - 1; P++)
    {

    /* Find pivot element */  

      for (K = P + 1; K <= N; K++)
      {
        if ( fabs(A[Row[K-1]][P-1]) > fabs(A[Row[P-1]][P-1]) )
        {
        /* Switch the index for the p-1 th pivot row if necessary */ 
           T      = Row[P-1];
           Row[P-1] = Row[K-1];
           Row[K-1] = T;
        }
      } /* End of simulated row interchange */

        if (A[Row[P-1]][P-1] == 0)
        {   
           printf("The matrix is SINGULAR !\n");
           printf("Cannot use algorithm --> exit\n");
           exit(1);
        }

     /* Form multiplier */

     for (K = P + 1; K <= N; K++)
     { 
       M = A[Row[K-1]][P-1] / A[Row[P-1]][P-1];
      
       /* Eliminate X_(p-1) */

       for (C = P + 1; C <= N + 1; C++)
       {
          A[Row[K-1]][C-1] -= M * A[Row[P-1]][C-1];
       }
     } 

  } /* End of upper-triangularization routine */


    if( A[Row[N-1]][N-1] == 0)
    {
      printf("The matrix is SINGULAR !\n");
      printf("Cannot use algorithm --- exit\n");
      exit(1);
    }

    /* Start the back substitution */
   
    X[N-1] = A[Row[N-1]][N] / A[Row[N-1]][N-1];

    for (K = N - 1; K >= 1; K--)
    {
      SUM = 0;
      for (C = K + 1; C <= N; C++)
      {
           SUM += A[Row[K-1]][C-1] * X[C-1];   
      }

      X[K-1] = ( A[Row[K-1]][N] - SUM) / A[Row[K-1]][K-1];

    }  /* End of back substitution */

    /* Output */

    for( K = 1; K <= N; K++)  printf("X[%d] = %lf\n", K, X[K-1]); 
    

} 

