/*
---------------------------------------------------------------------------
 Algo3-4.c   C program for implementing Algorithm 3.4
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 3.4 (Jacobi Iteration).
 Section   3.7, Iterative Methods for Linear Systems, Page 186
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------

 Algorithm 3.4 (Jacobi Iteration).

 To solve the linear system  AX = B  by starting with  P_0 = 0
 and generating a sequence  {P_K}  that converges to the
 solution  P  (i.e., AP = B).  A sufficient condition for the
 method to be applicable is that  A  is diagonally dominant.

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

/* -------------------------------------------------------- */

/*  Main program for algorithm 3.4  */

/*  remember : in C the fields begin with element 0  */

#define Limit  20

void main(void)
{
    double Tol = 10E-6;              /* Tolerance                   */
    double Sep  = 1.0;               /* Initialize                  */
    int K    = 1;                    /* Counter for iterations      */
    int Max  = 99;                   /* Maximum number of iterat.   */
    int Cond = 1;                    /* Condition of matrix         */
    int R, C, J;                     /* Loop counters               */
    double A[Limit][Limit];          /* A in  AX = B ,  INPUT       */
    double B[Limit];                 /* B in  AX = B ,  INPUT       */
    int N;                           /* Dimension of A, INPUT       */   
                                     /* = Number of equations       */
    double Row;                      /* Variable in dominance check */
    double P[Limit];                        
    double Pnew[Limit];
    double Sum;

    printf("Try the examples on page 188 of the book !\n");
    printf("------------------------------------------\n");
    do  /* force proper input */
    {
      printf("Please enter number of equations [Not more than %d]\n",Limit);
      scanf("%d", &N);
    } while( N > Limit);

    printf("You say there are %d equations.\n", N);
    printf("-----------------------------------------------------\n");
    printf("From  AX = B enter components of vector B one by one:\n");

    for (R = 1; R <= N; R++)
    {
        printf("Enter %d st/nd/rd component of vector B\n", R);
        scanf("%lf", &B[R-1]);
    }
    printf("-----------------------------------------------------\n");
    printf("From  AX = B enter elements of A row by row:\n");
    printf("-----------------------------------------------------\n");

    for (R = 1; R <= N; R++)
    {
        for (J = 1; J <= N; J++)
        {
             printf(" For row %d enter element %d please :\n", R, J);
             scanf("%lf", &A[R-1][J-1]);
        }
    }

    /* Check for diagonal dominance. */
    for (R = 1; R <= N; R++)
    {
       Row = 0.0;    
       for (C = 1; C <= N; C++) Row += fabs(A[R-1][C-1]);
       if( Row >= 2.0 * fabs(A[R-1][R-1]) ) Cond = 0; 
    }
        
    if( Cond == 0 ) 
    {
        printf("The matrix is not diagonally dominant.\n");
        printf("Cannot apply this algorithm ---> exit\n");
        exit(1);
    }

    /* Initialize : allow user to do this in order to create a
                    sequence of P_k values  */
    for ( J = 0; J < N; J++)   
    {
     P[J]    = 0;
     Pnew[J] = 0;
    }
      
    /* Perform Jacobi iteration */
    while( (K < Max) && (Sep > Tol) )
    {
     for (R = 1; R <= N; R++)
     {
        Sum = B[R-1];
        for (C = 1; C <= N; C++) if(C != R) Sum -= A[R-1][C-1] * P[C-1];
        Pnew[R-1] = Sum / A[R-1][R-1];
     }
     /* Convergence criterion */
     Sep = 0;
     for (J = 1; J <= N; J++) Sep += fabs(Pnew[J-1] - P[J-1]);
     /* Update values and increment the counter */
     for (J = 1; J <= N; J++) P[J-1] = Pnew[J-1];
     K++;
    }

     /* output */
     if(Sep < Tol) printf("The solution to the linear system is :\n");
     else          printf("Jacobi iteration did not converge:\n");

     for (J = 1; J <= N; J++) printf("P[%d] = %lf\n", J, P[J-1]);

} 

