/*
---------------------------------------------------------------------------
 Algo4-1.c   C program for implementing Algorithm 4.1
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 4.1 (Evaluation of a Taylor Series).
 Section   4.1, Taylor Series and Calculation of Functions, Page 203
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 4.1 (Evaluation of a Taylor Series).

                        (k)              (k)                        k
                inf    f   (x_0)(x - x_0)        inf   D(k)(x - x_0)
       P(x)  =  Sum   ----------------------  =  Sum ----------------
                j=0             k!               k=0       k!

 by computing a partial sum with at most N terms.

---------------------------------------------------------------------------
*/

#define N 100   /* Maximum degree */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>


/*  Main program for algorithm 4.1  */

    void main()

{
    int K,M;                   /* Loop counter                    */
    double Tol = 1E-7;         /* Termination criterion           */
    double X0;                 /* Point of expansion              */
    double D[N];               /* Derivatives of f(x)             */
    double Close = 1;          /* Closeness of consecutive points */
    double Sum;                /* Taylor Series sum               */
    double Prod;               /* Temporary variable              */
    double X;                  /* Independent variable            */
    double Term;

    printf("-------------------------------------------\n");
    printf("Maximum degree is 100. Successful if consecutive :\n");
    printf("points are closer then Tol = %lf\n", Tol);
    printf("===========================================\n");

    printf("Please enter Point of expansion :\n");
    printf("[Remark :This is very often 0]:\n");
    scanf("%lf",&X0);
    printf("Point of expansion is : %lf\n",X0);
    printf("--------------------------------------------\n");
    do
    {
     printf("Please enter degree of Taylor Series ( < 100 ) !\n");
     scanf("%d",&M);
    } while(M > 100);

    printf("Degree of Taylor Series is : %d\n", M);
    printf("--------------------------------------------\n");

    printf(" Now we need the values of the derivatives.\n");
    printf(" Example: exp(x) has at expansion point x_0 = 0 as\n"); 
    printf("        : derivative always the value 1 \n"); 

    for(K = 0; K <= M; K++)
    {
     printf(" Enter Derivative No. %d of f(x) at point of expansion !\n",K);
     scanf("%lf",&D[K]);
    }

    K    = 0;        /* Initialize the counter  */
    Sum  = D[0];     /* Initialize the variable */
    Prod = 1.0;      /* Variable that holds (x - x_0)^k  / k! */

    printf("Please enter the independent variable X :\n");
    scanf("%lf",&X);

    if(X == X0) Close = 0;

    while( (Close >= Tol) && (K < M) )
    {
        K++;

        Prod = Prod * (X - X0) / K;

        while( (D[K] == 0) && (K < M) )
        {
            K++;
            Prod = Prod * (X - X0) / K;
        }

        Term = D[K] * Prod;
        if(Term != 0) Close = fabs(Term);

        Sum += Term;
    }

    printf("----------------------------------------------\n");

    if( (Close < Tol) && (K <= M) )
    {
      printf("The sum of the Taylor polynomial is %lf\n", Sum);
      printf("Consecutive partial sums are closer than: %lf\n", Close);
    }
    else
    {
      printf("The current partial sum is: %lf\n", Sum);
      printf("Convergence has NOT been achieved !\n");
    }

    printf("----------------------------------------------\n");


}   /* End of main program */

