/*
---------------------------------------------------------------------------
 Alog4-2.c   C program for implementing Algorithm 4.2
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 4.2 (Polynomial Calculus).
 Section   4.2, Introduction to Interpolation, Page 212
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 4.2 (Polynomial Calculus).

 To evaluate the polynomial  P(x), its derivative  P'(x), and
 its integral  INT(P(x)dx)  by performing synthetic division.

---------------------------------------------------------------------------
*/

/* For each task a space-consuming and a space-saving version
   will run.  */

#define M 100   /* Maximum degree */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>


/*  Main program for algorithm 4.2  */

    void main()

{
    int K,N;                   /* Loop counter; degree of polyn.  */
    double A[M];               /* Coefficients of P(x)            */
    double B[M];               /* B[0] will be value of P(x)      */
    double D[M];               /* D[0] will be P'(x)              */
    double I[M];               /* I[0]lwill be Int of P(x)        */
    double Poly;               /* Will be value of P(x)           */
    double Deriv;              /* Will be P'(x)                   */
    double Integ;              /* Will be Int of P(x)             */
    double C;                  /* Constant of integration         */
    double X;                  /* Independent variable            */


    printf("Both, normal and space-saving version will be used :\n");
    printf("===========================================\n");
    printf("Maximum degree is 100 ! :\n");
    printf("===========================================\n");

    do
    {
     printf("Please enter degree of polynomial ( < 100 ) !\n");
     scanf("%d",&N);
    } while(N > 100);
    printf("--------------------------------------------\n");

    printf("Please enter CONSTANT OF INTEGRATION :\n");
    scanf("%lf",&C);
    printf("Constant of Integration is: %lf\n",C);
    printf("--------------------------------------------\n");

    for(K = 0; K <= N; K++)
    {
       printf("Give Coefficient Number %d\n",K);
       scanf("%lf",&A[K]);
       printf("Coefficient Number %d is : %lf\n",K,A[K]);
    }
    printf("--------------------------------------------\n");

    printf("Please enter Independent Variable X :\n");
    scanf("%lf",&X);
    printf("The independent variable is : %lf\n",X);
    printf("--------------------------------------------\n");
    printf("\n");

    /* Algorithm to evaluate P(x) */

    B[N] = A[N];
    for(K = N - 1; K >= 0; K-- )  B[K] = A[K] + B[K+1] * X;
    printf("The value of P(x) is : %lf\n",B[0]);
    printf("----------------------------------------------\n");


    /* space-saving version : */

    Poly = A[N];
    for(K = N - 1; K >= 0; K-- )Poly = A[K] + Poly * X;
    printf("Space-saving version says :\n");
    printf("The value of P(x) is : %lf\n",Poly);
    printf("----------------------------------------------\n");

/*================================================================ */

    /* Algorithm to evaluate P'(x) */

    D[N-1] = N * A[N];
    for(K = N - 1; K >= 1; K-- )  D[K-1] = K * A[K] + D[K] * X;
    printf("The value of P'(x) is %lf\n", D[0]);
    printf("----------------------------------------------\n");


    /* Space-saving version : */

    Deriv = N * A[N];
    for(K = N - 1; K >= 1; K--) Deriv = K * A[K] + Deriv * X;
    printf("Space-saving version says :\n");
    printf("The value of P'(x) is %lf\n", Deriv);
    printf("----------------------------------------------\n");

/*================================================================ */

    /* Algorithm to evaluate  Int( P(x) ) */

    I[N+1] = A[N]/(N+1);
    for(K = N; K >= 1; K-- ) I[K] = A[K-1]/K + I[K+1] * X;
    I[0] = C + I[1]*X;
    printf("The value of I(x) is : %lf\n", I[0]);
    printf("----------------------------------------------\n");

    /* Space-saving version : */

    Integ = A[N]/(N+1);
    for(K = N; K >=1; K--) Integ = A[K-1]/K + Integ * X;
    Integ = C + Integ*X;
    printf("Space-saving version says :\n");
    printf("The value of I(x) is : %lf\n", Integ);
    printf("----------------------------------------------\n");

}   /* end main program */

