/*
---------------------------------------------------------------------------
 Algo4-5.c   C program for implementing Algorithm 4.5
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 4.4 (Nested Multiplication with Multiple Centers).
 Section   4.4, Newton Polynomials, Page 234
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 4.4 (Nested Multiplication with Multiple Centers).

 To evaluate the Newton form of a polynomial 

 P(x)  =  a_0 + a_1*(x - x_0) + a_2*(x - x_0)*(x - x_1)
      
          +  a_3*(x - x_0)*(x - x_1)*(x - x_2)
 
          + ... + a_N*(x - x_0)*(x - x_1)...(x - x_(N-1)),

 with multiple centers  x_0, x_1, ... x_(N-1).

---------------------------------------------------------------------------
*/

/* main for algo4-34c  page 233 */

/* if compiling with the djgpp C-compiler then include 
   the library : libpc with  -lpc
   Example :  gcc algo4-4.c -lm -lpc                      */
   

#include<math.h>
#include<stdio.h>
#include<stdlib.h>

#define MAX 20

void main(void)
{

int N;                         /* Degree of P(x)  :  INPUT         */
int K;                         /* Loop counters                    */
double X[MAX];                 /* INPUT : Abscissas of the points  */
double A[MAX+1];               /* INPUT : Coefficients of P(x)     */
double Sum;                    
double T;                      /* Independent variable             */
int  q;                        /* For comparison to end input loop */           
       
    

    printf("-----------------------------------------------------\n");
    printf("Try examples on page 235 !\n");

    do
    {
      printf("Please enter degree of P[x] (Not > 20):\n");
      scanf("%d", &N);
    } while( N > MAX);

    printf("-----------------------------------------------------\n");
    printf("Degree of P(x) as entered is : %d\n", N);
    printf("-----------------------------------------------------\n");

    printf("Now enter all the Coefficients of P(x) :\n");
    printf("Not more then %d = [0 - %d] !\n", N+1, N);

    for( K = 0; K <= N; K++) 
    {
     printf("Please enter a_%d\n", K);
     scanf("%lf", &A[K]);
     printf("You entered : a_%d = %lf\n", K, A[K]); 
    }

    printf("----------------------------------------------\n");
    printf("Now enter all the Centers of P(x) :\n");
    printf("Not more then %d = [0 - %d] !\n", N, N-1);

    for( K = 0; K <= N-1; K++) 
    {
     printf("Please enter x_%d\n", K);
     scanf("%lf", &X[K]);
     printf("You entered : x_%d = %lf\n", K, X[K]); 
    }

     
    printf("-----------------------------------------------------\n");
    printf("Now comes a LOOP for input of the independent variable.\n");       
    printf("-----------------------------------------------------\n");
    for(; ; )
    {
     fflush(stdin);
     printf("TYPE  q  IF you want to STOP, else any other key.\n");
     q = getchar();

     if( q == 'q') break ;

     printf("Ok. You want to continue, so.....\n");
     printf("Please enter the independent variable X :\n");
     scanf("%lf", &T);

     Sum = A[N];   /* initialize variable */

     /* Implement nested multiplication */
     for( K = N - 1; K >= 0; K--) Sum = Sum * (T - X[K]) + A[K]; 

     /* output */
     printf("--------------------------------------------------\n");
     printf("The value of Polynomial P(x) is : %lf\n", Sum);
    }

}

