/*
---------------------------------------------------------------------------
 Algo4-5.c   C program for implementing Algorithm 4.5
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 4.5 (Newton Interpolation Polynomial).
 Section   4.4, Newton Polynomials, Page 234
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 4.5 (Newton Interpolation Polynomial).

 To construct and evaluate the Newton Polynomial of degree <= N
 that passes through  (x_k,y_k) = (x_k, f(x_k))  for k = 0,1,....,N:

 P(x)  =  d_(0,0) + d_(1,1)*(x - x_0) + d_(2,2)*(x - x_0)*(x - x_1)
      
                  + ... + d_(N,N)*(x - x_0)*(x -x1)...(x - x_(N-1))

 where 

                                   d_(k,j-1) - d(k-1,j-1)
 d_(k,0)  =  y_k  and  d_(k,j) =  ------------------------ .
                                         x_k - x_(k-j)

---------------------------------------------------------------------------
*/

/* main for algo4-5.c  page 234 */
/* if compiling with the djgpp C-compiler then include 
   the library : libpc with  -lpc
   Example :  gcc algo4-5.c -lm -lpc                      */
   

#include<math.h>
#include<stdio.h>
#include<stdlib.h>

#define MAX 20

void main(void)
{

int N;                         /* Degree of P(x)                   */
int K, J;                      /* Loop counters                    */
double X[MAX], Y[MAX];         /* INPUT : Abscissas of the points  */
                               /* INPUT : Ordinates of the points  */
double Sum;                    
double D[MAX][MAX];            /* Array                            */
double XD;                     /* Independent variable             */
int  q;                        /* For comparison to end input loop */           
       

    do
    {    
      printf("Please enter degree of P[x] (Not > 20)\n");
      scanf("%d", &N);
    } while(N > MAX);
    printf("-----------------------------------------------------\n");
    printf("Degree of P[x] as entered is : %d\n", N);
    printf("-----------------------------------------------------\n");

    printf("Now enter all the x and y values.\n");
    printf("Not more then %d = [0 - %d] !\n", N+1, N);

    for( J = 0; J <= N; J++) 
    {
     printf("Please enter : x_%d, y_%d\n", J,J);
     scanf("%lf, %lf", &X[J], &Y[J]);
     printf("You entered : x,y : %lf %lf\n",X[J], Y[J]);
    }


/*  Store ordinates in column 0 of the array D[K][J]  */ 
    for( K = 0; K <= N; K++) D[K][0] = Y[K];

    for( J = 1; J <= N; J++) 
    {
        for( K = J; K <= N; K++) 
        {
            D[K][J] = ( D[K][J-1] - D[K-1][J-1] ) / ( X[K] - X[K-J] );
        }
    }
     
    printf("-----------------------------------------------------\n");
    printf("Now comes a LOOP for input of the independent variable.\n");       
    printf("-----------------------------------------------------\n");
    for(; ; )
    {
     fflush(stdin);
     printf("TYPE  q  IF you want to STOP, else any other key.\n");
     q = getchar();

     if( q == 'q') break ;

     printf("Ok. You want to continue, so.....\n");
     printf("Please enter the independent variable X :\n");
     scanf("%lf", &XD);

     Sum = D[N][N];
     for( K = N-1; K >= 0; K--) Sum = Sum * (XD - X[K]) + D[K][K];

     printf("The value of the Newton Polynomial P[%lf] is %lf:\n", XD, Sum);
     printf("Based on the interpolation nodes:\n");
     printf("-----------------------------------------------------\n");
     printf(" No.    x_No.        y_No.\n");
     for(K = 0; K <= N; K++) printf("%d  %lf  %lf\n",K, X[K], Y[K]);
    }

}

