/*
---------------------------------------------------------------------------
 Algo4-6.c   C program for implementing Algorithm 4.6
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 4.6 (Chebyshev Approximation).
 Section   4.5, Chebyshev Polynomials, Page 246
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 4.6 (Chebyshev Approximation).
 To  construct and evaluate the Chebyshev interpolating polynomial
 degree  N  over the interval [-1,1], where

                 N
       P(x)  =  Sum  c_j T_j(x)
                j=0

 is based on the nodes


                     ( 2*k  +1 ) * Pi
        x_k  =  cos( ----------------- ).
                         2* N + 1

---------------------------------------------------------------------------
*/

/* User has to supply a function named : ffunction
   An example is included in this program */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>


/*  define prototype for USER-SUPPLIED function f(x)  */

    double ffunction(double x);


/*  EXAMPLE for "ffunction" . See page 246 example 4.16  */

    double ffunction(double x)

    {
        return ( exp(x) );
    }

/* -------------------------------------------------------- */

/*  Main program for algorithm 4.6  */

    void main()

{
    int J, K;                  /* loop counter                   */
    int N;                     /* INPUT: Degree of polynomial    */
    double Pi = 3.1415926535;  /* Pi                             */
    double D;                  /* holds part of the node expr.   */
    double X[10];              /* nodes                          */
    double Y[10];              /* function values                */
    double C[10];              /* coefficients                   */
    double T[10];              /* Chebyshev polynomial           */
    double Z;                  /* sum for coefficients           */
    double P;                  /* sum of Cheb. polynomials       */
    double xx;                 /* independent variable           */


    printf("--------Example 4.16----page 246--------------\n");
    printf("Please enter the degree N of the polynomial :\n");
    printf("EXAMPLE: N = 3, so type: 3\n");
    scanf("%d",&N);
    if(N > 9)
    {
     printf(" Such a high degree is suspicious. Terminating. Sorry\n");
     exit(0);
    }

    printf("The degree of the polynomial is N =  %d\n", N);
    printf("----------------------------------------------\n");

    D = Pi / (2.0 * N + 2);

    for(K = 0; K <= N; K++)
    {
        X[K] = cos( (2.0 * K + 1) * D ); /* Make the nodes            */
        Y[K] = ffunction(X[K]);          /* Store the function values */
        C[K] = 0.0;                      /* Initialize the array      */
    }

    for (K = 0; K <= N; K++)
    {
        Z = (2.0 * K + 1) * D;           /* Calculate the sum for the */
        for(J = 0; J <= N; J++)          /* coefficients              */
        {
            C[J] = C[J] + Y[K] * cos(J*Z);
        }

    }

    C[0] = C[0] / (N + 1);

/*  printf("coeff 1 %lf\n",C[0]);  coefficient no. 1 of Pylon */

    /* Multiple of sum forms coefficient */

    for(J = 1; J <= N; J++)
    {
         C[J] = 2.0 * C[J] / (N + 1);

/*  printf("coeff %d is %lf\n",J,C[J]); other coefficients */

    }

    printf("--------Example 4.16----page 246--------------\n");
    printf("Please enter the independent variable X :\n");
    printf("EXAMPLE: X = 2.5 so type: 2.5\n");
    scanf("%lf",&xx);
    printf("The independent variable X =  %lf\n", xx);
    printf("----------------------------------------------\n");

    T[0] = 1.0;
    T[1] = xx;

    if( N > 1)
    {
        for(J = 1; J <= N-1; J++)             /* Recursively generate   */
        {
         T[J+1] = 2.0 * xx * T[J] - T[J-1];   /* the Chebyshev          */
        }                                     /* coefficient polynomials*/
    }

    P = C[0] * T[0];

    /* Evaluate Chebyshev polynomial */

    for(J = 1; J <= N; J++)  P = P + C[J] * T[J];

    printf("----------------------------------------------\n");
    printf(" The value of the Chebyshev \n");
    printf(" polynomial  P(X) is : %lf\n", P );
    printf("----------------------------------------------\n");

}   /* End of main programm */

