/*
---------------------------------------------------------------------------
 Algo6-1.c   C program for implementing Algorithm 6.1
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 6.1 (Differentiation Using Limits).
 Section   6.1, Approximating the Derivative, Page 326
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------

 Algorithm 6.1 (Differentiation Using Limits).
 To approximate  f'(x)  numerically by generating the sequence


                           -k               -k
                f(x + h * 2  ) - f(x - h * 2  )
     f'(x)  =  ---------------------------------    for k =0,..,.n
                              -k+1
                         h * 2


 until |D_(n+1) - D_n| => |D_n - D_(n-1)| < Tol, which is an attempt

 to find the best approximation f'(x) approx. D_n  .

---------------------------------------------------------------------------
*/

/* User has to supply a function named : ffunction
   An example is included in this program */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define Max 20   /* Maximum number of terms        */


/*  define prototype for USER-SUPPLIED function f(x)  */

    double ffunction(double x);


/*  EXAMPLE for "ffunction" . See page 325 example 6.3  */

    double ffunction(double x)

    {
        return ( cos(x) );
    }

/* -------------------------------------------------------- */

/*  Main program for algorithm 6.1  */

    void main()

{
    int N;                     /* Counter                        */
    double Tol = 1E-5;         /* Convergence criterion          */
    double H = 1.0;            /* Initial Step size              */
    double D[Max + 1];         /* Approximations                 */
    double E[Max + 1];         /* Error estimate                 */
    double R[Max + 1];         /* Relative Error                 */
    double X;                  /* INPUT : Abscissa for f'(x)     */
    double Danger = 1E+100;    /* Border for errors              */

    printf("--------Example 6.3----page 325--------------\n");
    printf("Please enter the abscissa  x  for f'(x)\n");
    printf("EXAMPLE: x = 0.8, so type: 0.8\n");
    scanf("%lf",&X);
    printf("The abscissa  x =  %lf \n", X);
    printf("----------------------------------------------\n");

    /* Compute the quotient */

    D[0] = 0.5 * ( ffunction(X + H) - ffunction(X - H) ) / H;

    /* Computer D_1 and D_2 */
    
    for (N = 1; N <= 3; N++)
    {
        H = H / 2.0;         /* Reduce step size */

        /* Compute the quotient */

        D[N] = 0.5 * ( ffunction(X + H) - ffunction(X - H) ) / H;

        /* Error estimate and relative error */

        E[N] = fabs( D[N] - D[N-1] );
        R[N] = 2.0 * E[N] / ( fabs( D[N] ) + fabs( D[N-1] ) + Tol );
    }

    N = 1;   /* Initialize counter */

    while( ( (E[N] > E[N+1]) || (R[N] >= Tol) ) && N < Max )
    {
        H = H / 2.0;    /* Reduce step size */

        /* Compute the quotient */

        D[N+2] = 0.5 * ( ffunction(X + H) - ffunction(X - H) ) / H;

        /* Error estimate and relative error */

        E[N+2] = fabs( D[N+2] - D[N+1] );
        R[N+2] = 2.0 * E[N+2] / ( fabs(D[N+2]) + fabs(D[N+1]) + Tol );

/* check if Error or RelErr grow and prevent floating point exception */

     if ( E[N+2] > Danger || R[N+2] > Danger)
     {
        printf("Error and/or Relative Error grow(s) too big !\n");
     }
        N++;   /* Increment the counter */
    }

    printf("-------- Differentiation Using Extrapolation-------\n");

    printf(" The approximate value of f'(x) using Limits\n");
    printf(" is = %lf\n", D[N-1] );
    printf(" The accuracy is + - %lf\n", E[N-1] );

    printf("----------------------------------------------\n");

}   /* End of main program */

