/*
---------------------------------------------------------------------------
 Algo6-2.c   C program for implementing Algorithm 6.2
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 6.2 (Differentiation Using Extrapolation).
 Section   6.1, Approximating the Derivative, Page 327
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------

 Algorithm 6.2 (Differentiation Using Extrapolation).
 To  approximate  f'(x)  numerically by generating a table of
 approximate  D(J,K)  for  K <= J, and using  f'(x)  approx.  D(N,N)
 as the final answer. The approximations  D(J,K)  are stored in
 a lower-triangular matrix.

                            -J             -J
                   f(x + h*2  ) - f(x - h*2  )
       D(J,0)  =  -----------------------------
                                 -J+1
                            h * 2

 and the elements in row J are :

                        D(J,K-1) - D(J-1,K-1)
 D(J,K)  =  D(J,K-1) + -----------------------
                               K
                              4  - 1
 for 1 <= K <= J.

---------------------------------------------------------------------------
*/

/* User has to supply a function named : ffunction
   An example is included in this program */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>


/*  define prototype for USER-SUPPLIED function f(x)  */

    double ffunction(double x);


/*  EXAMPLE for "ffunction" . See page 325 example 6.3  */

    double ffunction(double x)

    {
        return ( cos(x) );
    }

/* -------------------------------------------------------- */

/*  Main program for algorithm 6.2  */

    void main()

{
    int J = 1, K;              /* loop counter                   */
    int N;                     /* remembers column,row           */
    double Tol    = 1E-5;      /* termination                    */
    double Delta  = 1E-7;      /* criterion                      */
    double Error  = 1.0;       /* initialize the variables       */
    double RelErr = 1.0;       /* relative error                 */
    double H = 0.01;           /* step size                      */
    double D[13][13];          /* approximations                 */
    double X;                  /* INPUT : Abscissa for f'(x)     */
    double Temp;               /* for temporary storage          */
    double MAX = 1E+100;       /* control for Termination        */


    printf("--------Example 6.3----page 325--------------\n");
    printf("Please enter the abscissa  x  for f'(x)\n");
    printf("EXAMPLE: x = 0.8, so type: 0.8\n");
    scanf("%lf",&X);
    printf("The abscissa  x =  %lf \n", X);
    printf("----------------------------------------------\n");

    /* Compute the quotient */

    D[0][0] = 0.5 * ( ffunction(X + H) - ffunction(X - H) ) / H;

    while ( (RelErr > Tol) && (Error > Delta) && (J <= 12) )
    {

     H = H / 2.0;      /* Reduce the step size */

     /* Compute the quotient */

     D[J][0] = 0.5 * ( ffunction(X + H) - ffunction(X - H) ) / H;

     for (K = 1; K <= J; K++)
     {
        Temp    = ( D[J][K-1] - D[J-1][K-1] ) / ( pow(4,K) - 1);
        D[J][K] = D[J][K-1] + Temp;
     }
     
     Error = fabs( D[J][J] - D[J-1][J-1] );   /* Error estimate */
     RelErr = 2.0 * Error / ( fabs(D[J][J]) + fabs(D[J-1][J-1]) + Tol );

/* check if Error or RelErr grow and prevent floating point exception */

     if ( Error > MAX || RelErr > MAX)
     {
      printf("Error and/or RelErr grow(s) too big !\n");
     }

    J++;
    N = J;
    }

    printf("-------- Differentiation Using Extrapolation-------\n");

    printf(" The approximate value of f'(x) using Richardson's\n");
    printf(" extrapolation is = %lf\n", D[N-1][N-1] );
    printf(" The accuracy is + - %lf\n", Error );

    printf("----------------------------------------------\n");

}   /* End of main program */

