/*
---------------------------------------------------------------------------
 Algo6-3.c   C program for implementing Algorithm 6.3
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 6.3 (Differentiation Based on N+1 Nodes).
 Section   6.2, Numerical Differentiation Formulas, Page 342
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 6.3 (Differentiation Based on N + 1 nodes).

 To approximate f'(x) numerically by constructing the 
 Nth-degree Newton polynomial 

 P(x)  =  a_0 + a_1*(x -x_0) + a_2*(x-x_0)*(x-x_1)
                             + a_3*(x-x_0)*(x-x_1)*(x-x_2)
              + ... + a_N*(x-x_0)*(x-x_1) ... (x-x_(N-1)

 and using  f'(x_0)  approx.  P'(x_0)  as the final answer. 

 The method must be used at x_0. The points can be 

 rearranged {x_k, x_0, ... , x_(k-1), x_(k+1), ... , x_N}

 to compute  f'(x_k)  approx  P'(x_k).

---------------------------------------------------------------------------
*/

/* main for algo6-3.c  page 342 */
   

#include<math.h>
#include<stdio.h>
#include<stdlib.h>

#define MAX 20

void main(void)
{

int N;                         /* Degree of P(x)                   */
int K, J;                      /* Loop counters                    */
double X[MAX], Y[MAX];         /* INPUT : Abscissas of the points  */
                               /* INPUT : Ordinates of the points  */
double Sum;                    
double A[MAX];                 /* Array                            */
double Df, XNull;              /* Independent variable             */
double Prod;

    printf("-----------------------------------------------------\n");
    printf("----Differentiation Based on N + 1 nodes-------------\n");
    printf("-----------------------------------------------------\n");
    printf("\n");
    do
    {    
      printf("Please enter degree of P[x] (Not > 20 )\n");
      printf("( = Number of points - 1 )\n");
      scanf("%d", &N);
    } while(N > MAX);
    printf("-----------------------------------------------------\n");
    printf("Degree of P[x] as entered is : %d\n", N);
    printf("-----------------------------------------------------\n");

    printf("Now enter all the x and y values.\n");
    printf("Not more then %d = [0 - %d] !\n", N+1, N);

    for( J = 0; J <= N; J++) 
    {
     printf("Please enter : x_%d, y_%d\n", J,J);
     scanf("%lf, %lf", &X[J], &Y[J]);
     printf("You entered : x,y : %lf %lf\n",X[J], Y[J]);
    }

    /*  Initialize the coefficient array  */ 

    for( K = 0; K <= N; K++) A[K] = Y[K];

    /*  Compute the divided differences */

    for( J = 1; J <= N; J++)
    {
       for( K = N; K >= J; K--) 
       {
          A[K] = ( A[K] - A[K-1] ) / ( X[K] - X[K-J] );
       }
    }

     /* Compute the derivative at x_0 ;
        first initialize the variables */

     XNull    = X[0];              
     Df       = A[1];
     Prod     = 1.0;

     for( K = 2; K <= N; K++)
     {
         Prod = Prod * ( XNull - X[K-1] );
         Df   = Df +    Prod * A[K] ;
     }

     printf("\n");
     printf("-----------------------------------------------------\n");
     printf("The numerical (approx.!) derivative f'(x_0) = %lf\n", Df);
     printf("-----------------------------------------------------\n");

}

