/*
---------------------------------------------------------------------------
 Algo7-2.c   C program for implementing Algorithm 7.2
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 7.2 (Composite Simpson Rule).
 Section   7.2, Composite Trapezoidal and Simpson's Rule, Page 365
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------

 Algorithm 7.2 (Composite Simpson Rule).  To  approximate the integral

     b
     /              h                      2h   M-1
     | f(x)dx   =   - * [ f(A) - f(B)]  +  -- * sum  f(x_(2k))
     /              3                       3   k=1
     a
                         4h    M
                      +  -- * sum  f(x_(2k-1))
                          3   k=1

 by sampling f(x) at the 2M + 1  equally spaced points

 x_k = A + h*k  for  k = 0,1,2...,2M. 

 Notice that  x_0 = A  and  x_2M = B.

---------------------------------------------------------------------------
*/

/* User has to supply a function named : ffunction
   An example is included in this program */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>


/*  define prototype for USER-SUPPLIED function f(x)  */

    double ffunction(double x);


/*  EXAMPLE for "ffunction"   */

    double ffunction(double x)

    {
        return ( 1 / ( 1 + pow(x,2) ) );
    }

/* -------------------------------------------------------- */

/*  Main program for algorithm 7.2  */

    void main()

{
    int K;                     /* loop counter                   */
    int M;                     /* INPUT : number of subintervals */
    double A,B;                /* INPUT : boundaries of integral */
    double H;                  /* Subinterval width              */
    double SumEven = 0;        /* approx. Even integral          */
    double SumOdd  = 0;        /* approx. Odd  integral          */
    double SUM     = 0;        /* approx. integral value         */
    double X;


    printf("----------------------------------------------\n");
    printf("Please enter the boundaries of the integral [A,B]\n");
    printf("EXAMPLE: A = -1 and B = 1, so type: -1 1\n");
    scanf("%lf %lf",&A, &B);
    printf("The boundaries of the integral are: %lf %lf\n",A, B);
    printf("----------------------------------------------\n");
    printf("Please enter the number of SUBINTERVALS.\n");
    scanf("%d",&M);

    printf("You say : %d subintervals.\n",M);

    H = (B - A)/(2*M);        /*  Subinterval width    */

    /* Loop for even subscripts */

    for ( K = 1; K <= M-1; K++ ) {
         X = A + H * 2 * K;
         SumEven = SumEven + ffunction(X);
    }

    /* Loop for odd subscripts */

    for ( K =  1; K <= M; K++ ) {
         X = A + H*( 2 * K - 1);
         SumOdd = SumOdd + ffunction(X);
    }

    SUM = H * ( ffunction(A) + ffunction(B) + 2*SumEven + 4*SumOdd ) / 3;


    printf("----------------------------------------------\n");

    printf(" The approximate value of the integral of f(X)\n");
    printf(" on the interval %lf %lf\n", A,B);
    printf(" using %d subintervals computed using the Simpson's\n",2*M);
    printf(" rule is : %lf\n",SUM);

    printf("----------------------------------------------\n");

}   /* End of main program */

