/*
---------------------------------------------------------------------------
 Algo7-4.c   C program for implementing Algorithm 7.4
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 7.4 (Romberg Integration).
 Section   7.3, Recursive Rules and Romberg Integration, Page 379
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------

 Algorithm 7.4 (Romberg Integration).  To  approximate the integral

     b
     /
     | f(x)dx   =   R(J,J);
     /
     a

 by generating a table of approximations  R(I,K)  for K <= I, and using
 R(J,J)  as the final answer.  The approximations  R(J,K)  are
 stored in a special lower-triangular matrix.  The elements  R(J,0)  of
 column 0 are computed using the sequential trapezoidal rule based on
  J
 2  subintervals of [A,B]; then R(J,K) is computed using Romberg's rule.

 The elements of row J are
                          R(J,K-1) - R(J-1,K-1)
  R(J,K)  =  R(J,K-1)  +  ---------------------     for 1 <= K <= J.
                                   K
                                  4   -1

 The algorithm is terminated in the J-th row when :

           |R(J,J) - R(J-1, J-1)| < Tol.

---------------------------------------------------------------------------
*/

/* User has to supply a function named : ffunction
   An example is included in this program */

#define MAX 25
#include<stdio.h>
#include<stdlib.h>
#include<math.h>


/*  define prototype for USER-SUPPLIED function f(x)  */

    double ffunction(double x);


/*  EXAMPLE for "ffunction"   */

    double ffunction(double x)

    {
        return ( 1 / sqrt(x) );
    }

/* -------------------------------------------------------- */

/*  Main program for algorithm 7.4  */

    void main()

{
    int P, K, J, KK;           /* loop counters                  */
    int M;                     /* number of subintervals         */
    int N;                     /* INPUT : number of times subintervals
                                          are bisected  */
    double Tol = 1E-7;         /* Termination criterion,         */
    double Close;              /*                                */
    double A,B;                /* INPUT : boundaries of integral */
    double H;                  /* Step width                     */
    double SUM     = 0;        /*                                */
    double R[MAX][MAX];        /* Array passed to the subroutine */

    printf("----------------------------------------------\n");
    printf("Please enter the boundaries of the integral [A,B]\n");
    printf("EXAMPLE: A = 0.04 and B = 1, so type: 0.04 1\n");
    scanf("%lf %lf",&A, &B);
    printf("The boundaries of the integral are: %lf %lf\n",A, B);
    printf("----------------------------------------------\n");
    printf("Maximum number of rows ? [ <= MAX]\n");
    scanf("%d",&N);

    if( N > MAX ) {
         printf(" :-() Maximum is %d ! Start again.\n", MAX);
         exit(1);
    }
    printf("You say : %d rows\n",N);

    M = 1;     /* Initialize the number of subintervals */
    H = B - A; /* Initialize the stepsize               */
    Close = 1; /* Initialize the variable               */
    J = 0;     /* Initialize the counter                */

    R[0][0] =  H * ( ffunction(A) - ffunction(B) ) /2; /* Compute one
                                                         trapezoid */

   while ( ( (Close > Tol) && (J < N) ) || (J < 4) ) {
     /* Do sequential calculations */
       J++;
       /* Compute new R(J,0)          */
       /* Sequential trapezoidal rule */

    H = H/2;     /* Reduce step size for J-th refinement */
    SUM = 0;
    for (P = 1; P <= M; P++) { /* Loop for odd subscripts */
        SUM = SUM + ffunction( A + H*(2 * P - 1) );
    }
        R[J][0] = R[J-1][0] / 2 + H * SUM;
        M = 2*M;     /* Update number of subintervals */


    /* Richardson's improvements */

      for(K = 1; K <= J; K++) {
       R[J][K] = R[J][K-1] + ( R[J][K-1] - R[J-1][K-1] )/( pow(4,K) -1 );
      }
    printf("The entries in row %d for Romberg integration are\n",J);

   for (KK = 1; KK <= J; KK++) printf("R(%d,%d) = %lf\n",J,KK,R[J][KK]);

    Close = fabs( R[J-1][J-1] - R[J][J] );
   } /* end of while-loop */
    printf("----------------------------------------------\n");

    printf(" The best approximation for value of the integral\n");
    printf(" using Romberg integration is = %lf\n", R[J][J]);

    printf("----------------------------------------------\n");

}   /* End of main program */

