/*
---------------------------------------------------------------------------
 Algo8-1.c   C program for implementing Algorithm 8.1
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 8.1 (Golden Search for a Minimum).
 Section   8.1, Minimization of a Function, Page 413
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------

 Algorithm 8.1 (Golden Search for a Minimum).

 To numerically approximate the minimum of  f(x)  on the interval 
 [a,b] by using a golden search. Proceed with the method only
 if  f(x)  is a unimodal function on the interval [a,b].

---------------------------------------------------------------------------
*/

/* User has to supply a function named : ffunction
   An example is included in this program */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>


/*  define prototype for USER-SUPPLIED function f(x)  */

    double ffunction(double x);


/*  EXAMPLE for "ffunction" : see page 420 No. 10  */

    double ffunction(double x)

    {
        return ( -sin(x) - sin(3 * x) / 3.0 );
    }

/* -------------------------------------------------------- */

/*  Main program for algorithm 7.1  */


void main(void)
{
    double Delta   = 10E-8;                /* Tolerance for interval width */
    double Epsilon = 10E-14;               /* Tolerance for |f(b) -f(a)|   */
    double Rone    = (sqrt(5.0)-1.0)/2.0;  /* Determine constants          */
    double Rtwo    = Rone * Rone;          /* for golden search            */
    double A, B;                           /* endpoints of Interval        */
    double YA, YB;                         /* function values at A and B   */
    double C, D;                           /* interior points              */
    double H;                              /* width of interval            */   
               
    double P, YC, YD, YP;                  /* minimum and function values  */   
 


    printf("Please enter endpoints of interval : A, B\n");
         printf("For present example type : 1, 3\n");
    scanf("%lf %lf", &A, &B);
         printf("A = %lf and B = %lf\n", A, B);    
    printf("\n");


    YA = ffunction(A);
    YB = ffunction(B);
    H  = B - A;
    C  = A + Rtwo * H;
    YC = ffunction(C);
    D  = A + Rone*H;
    YD = ffunction(D);

    while( fabs(YA - YB) > Epsilon || H > Delta)
    {
        if( YC < YD) 
        {
            B  = D;
            YB = YD;
            D  = C;
            YD = YC;
            H  = B - A;
            C  = A + Rtwo * H;
            YC = ffunction(C);
        }
        else
        {
            A  = C;
            YA = YC;
            C  = D;
            YC = YD;
            H  = B - A;
            D  = A + Rone * H;
            YD = ffunction(D);
        }
    }

    P  = A;
    YP = YA;
    if( YB < YA) 
    {
        P  = B;
        YP = YB;
    }

    printf("--------------------------------------------------\n");
    printf("The minimum of f(x) occurs at %lf \n", P);
    printf("The accuracy is +/- %lf\n", H);
    printf("The minimum of the function f(Minimum) is : %lf\n", YP);
    printf("--------------------------------------------------\n");
} 

