/*
---------------------------------------------------------------------------
 Algo9-2.c   C program for implementing Algorithm 9.2
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 9.2 (Heun's Method).
 Section   9.3, Heun's Method, Page 441
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 9.2 (Heun's Method).

 To approximate the solution of the initial value problem  y' = f(t,y)
 with  y(a) = y_0  over [a,b] by computing

                      h
 y_(k+1)  =  y_k  +  --- * [f(t_k,y_k) + f(t_(k+1),y_k+h*f(t_k,y_k))]
                      2

 for k = 0,1,...,M-1.  


 User has to supply a function named : ffunction
 An example is included in this program.

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define MAX 500


/*  define prototype for USER-SUPPLIED function f(x)  */

    double ffunction(double t, double y);


/*  EXAMPLE for "ffunction"   */

    double ffunction(double t, double y)

    {
        return ( (t - y) / 2.0 );
    }

/* -------------------------------------------------------- */

/*  Main program for algorithm 9.2  */

    void main(void)

{
    int J;                     /* Loop counter                   */
    int M;                     /* INPUT: Number of steps         */
    double A, B, Y[MAX];       /* Endpoints and inital value     */
    double H;                  /* Compute the step size          */
    double T[MAX]; 
    double K1, K2;             /* Function values                */
    double P;                  /* Euler predictor                */


    printf("----------- Heun's Method -------------------\n");
    printf("--------- Example on page 433 ---------------\n");
    printf("-See table 9.4 on page 440 for comparison with-\n");
    printf("-algorithm 9.1 (Euler's method)-\n");
    printf("Please enter endpoints of the interval [A,B]:\n");
    printf("For used Example type  : 0, 3\n");
    scanf("%lf, %lf", &A, &B);
    printf("You entered [%lf, %lf]\n", A, B);
    printf("-----------------------------------------\n");
    printf("Please enter number of steps: (Not more than 500 !)\n");
    scanf("%d",&M);
    if(M > MAX)
    {
    printf(" Not prepared for more than %d steps. Terminating. Sorry\n",MAX);
    exit(0);
    }

    printf("-----------------------------------------\n");
    printf("Please enter initial value Y[0] :\n");
    printf("For used Example type : 1\n");
    scanf("%lf", &Y[0]);
    printf("You entered Y[0] = %lf\n", Y[0]);

    /* Compute the step size */

    H    = (B - A) / M;      

    /* Initialize the variable */

    T[0] = A;


    for(J = 0; J <= M-1; J++)
    {
       K1 =  ffunction(T[J], Y[J]);      /* Function value at t_J          */
       P  =  Y[J] + H * K1;              /* Euler predictor for y_(J+1)    */
       T[J+1] = A + H * (J+1);           /* Compute the mesh point t_(J+1) */
       K2 = ffunction(T[J+1], P);            /* Function value at t_(J+1)  */
       Y[J+1] = Y[J] + H * (K1 + K2) / 2.0;  /* Trapezoidal corrector      */
    }

    /* Output */

    for ( J = 0; J <= M; J++)
    {
       printf("J = %d, T[J] = %lf, Y[J] = %lf\n", J, T[J], Y[J]);
    } 

}   /* End of main program */

